package rozdzial14c;

import javafx.animation.AnimationTimer;
import javafx.application.Application;
import javafx.scene.Scene;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.stage.Stage;
import java.time.Duration;
import java.time.Instant;

/**
 * @author Jacek Piechota
 <p>Kontrolujemy czas trwania animacji. Kontrolujemy płynność animacji
 przy użyciu interpolatora liniowego klasy {@code Linear} rozszerzającej klasę {@code Interpolator}.</p>
 <p>Po uruchomieniu na konsoli zobaczymy:</p>
 <p>{@code Czas trwania: 2008}</p>
 */
public class Listing14c_04b extends Application {
    private double radius;
    private Circle circle;
    private long duration;
    private Linear interp;

    @Override
    public void start(Stage stage) {
        StackPane root = new StackPane();
        circle = new Circle(radius);
        duration = 2000;//ms
        circle.setFill(Color.DARKCYAN);
        interp = new Linear();
        root.getChildren().add(circle);
        AnimationTimer timer = new MyTimer();
        timer.start();
        Scene scene = new Scene(root, 300, 300);
        stage.setTitle("AnimationTimer");
        stage.setScene(scene);
        stage.show();
    }

    private class MyTimer extends AnimationTimer {
        Instant start = Instant.now();
        @Override
        public void handle(long now) {
            long elapsed = Duration.between(start,
                    Instant.now()).toMillis();
            if(elapsed <= duration) {
                double radiusStart = 10.0;
                double radiusEnd = 80;
                double r = interp.interpolate(radiusStart, radiusEnd,
                        (double) elapsed / (double) duration);
                circle.radiusProperty().set(r);
            }else{
                radius = 10.0;
                circle.radiusProperty().set(radius);
                stop();
                Instant end = Instant.now();
                long millis = Duration.between(start, end).toMillis();
                System.out.println("Czas trwania: "+ millis);
            }
        }
    }

    public static void main(String[] args) {
        Application.launch(args);
    }
}